#####################################################################
# EJ(20170329)
# experimental indicators
#####################################################################

library(lme4)
library(ggplot2)
library(lattice)
library(latticeExtra)
library(reshape2)
library(parallel)
library(influence.ME)
library(xtable)
load("RData.nea")
load("RData.med")
source("funs.R")
options(stringsAsFactors=FALSE, width = 60)
theme_set(theme_bw())
sc <- scale_x_continuous(breaks=2003:2015)
th <- theme(axis.text.x  = element_text(angle=90, vjust=0.5))
nc <- 8
it <- 500
# to control de seed in mclapply
RNGkind("L'Ecuyer-CMRG")
set.seed(1234)

#====================================================================
# SSB trends for Cat03 stocks
#====================================================================
#--------------------------------------------------------------------
# load assessment data
#--------------------------------------------------------------------

#subset cat 03 stocks of EU Ecoregions in 2003-2015
df0 <- subset(isa, !(EcoRegion %in% c("Iceland Sea and Greenland Sea", "Barents Sea and Norwegian Sea", "Faroes")) & Category=="3" & Year>=2003 & Year<2016)

#--------------------------------------------------------------------
# Types of indices and units
#--------------------------------------------------------------------

df0$fStockSizeDescription <- as.factor(df0$StockSizeDescription)

df0$fStockSizeUnits <- as.factor(df0$StockSizeUnits)
levels(df0$fStockSizeUnits)[1] <- "NA"

df0$IndexType <- df0$fStockSizeDescription
levels(df0$IndexType)[1] <- "Abundance Index"
levels(df0$IndexType)[2] <- "Biomass Index"
levels(df0$IndexType)[3:8] <- "Relative Index"

# Adding two levels of index types to separate abundance index in millions
# and biomass index in tonnes (different scale and standardisation type than the others)
levels(df0$IndexType)[4] <- "Abundance index (millions)"
levels(df0$IndexType)[5] <- "Biomass index (tonnes)"
df0[df0$fStockSizeUnits=="millions",]$IndexType <- "Abundance index (millions)"
df0[df0$fStockSizeUnits=="tonnes",]$IndexType <- "Biomass index (tonnes)"

#--------------------------------------------------------------------
# biomass index model
#--------------------------------------------------------------------

df0$Year <- factor(df0$Year)
yrs <- levels(df0$Year)
nd <- data.frame(Year=factor(yrs))

df1 <- df0[df0$IndexType=="Biomass Index" & !is.na(df0$StockSize),]

# fit the GLMM
ifit03 <- glmer(StockSize ~ Year + (1|FishStock), data = df1, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap"))
#runDiagsME(ifit03, "FishStock", df1, "diagNEAIcat03BI.pdf", nc, nd) 

# bootstrap
stk <- unique(df1$FishStock)
ifit03.bs <- split(1:it, 1:it) 
ifit03.bs <- mclapply(ifit03.bs, function(x){
	stk <- sample(stk, replace=TRUE)
	df1 <- df0[0,] 
	for(i in stk) df1 <- rbind(df1, subset(df0, FishStock==i))
	fit <- glmer(StockSize ~ Year + (1|FishStock), data = df1,
    family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
	v0 <- predict(fit, re.form=~0, type="response", newdata=nd)
	if(length(fit@optinfo$conv$lme4)>0) v0[] <- NA
	v0
}, mc.cores=nc)

ifitm <- do.call("rbind", ifit03.bs)
#ifitm <- exp(log(ifitm)-mean(log(ifitm[,1]), na.rm=TRUE))
ifitq <- apply(ifitm, 2, quantile, c(0.025, 0.25, 0.50, 0.75, 0.975), na.rm=TRUE)
ifitq <- cbind(Year=as.numeric(yrs), as.data.frame(t(ifitq)))

# plot
png("figNEAIcat03.png", 600, 400)
ggplot(ifitq, aes(x=Year)) +
  geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), fill="gray", alpha=0.60) +
  geom_ribbon(aes(ymin = `25%`, ymax = `75%`), fill="gray", alpha=0.95) +
  geom_line(aes(y=`50%`)) + expand_limits(y=0) +
  geom_point(aes(x=Year[1], y=`50%`[1])) +
  geom_point(aes(x=Year[length(Year)], y=`50%`[length(`50%`)]), size=2) +
  ylab("Biomass index") + xlab("") +
  theme(legend.position = "none") + sc + th
dev.off()

# table
BItab <- t(ifitq)[-1,]
colnames(BItab) <- ifitq[,1]
write.csv(BItab, file="tabNEAIcat03BI.csv")

# StockTable
write.csv(df0, file='stockTableCat03.csv')

#====================================================================
# Recruitment model
# (in millions)
#====================================================================
#--------------------------------------------------------------------
# NEA
#--------------------------------------------------------------------
# ane-bisc reports R in weight not numbers
idx <- !is.na(saeu$Recruitment) & saeu$FishStock!='ane-bisc'
df0 <- saeu[idx,]
df0 <- transform(df0, Year=factor(df0$Year), Recruitment=Recruitment/1000)

yrs <- levels(df0$Year)
nd <- data.frame(Year=factor(yrs))
# fit
ifitr <- glmer(Recruitment ~ Year + (1|FishStock), data =  df0, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
runDiagsME(ifitr, "FishStock", df0, "diagNEAIR.pdf", nc, nd) 

# bootstrap
stk <- unique(df0$FishStock)
ifitr.bs <- split(1:it, 1:it) 
ifitr.bs <- mclapply(ifitr.bs, function(x){
	stk <- sample(stk, replace=TRUE)
	df1 <- df0[0,] 
	for(i in stk) df1 <- rbind(df1, subset(df0, FishStock==i))
	fit <- glmer(Recruitment ~ Year + (1|FishStock), data = df1, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
	v0 <- predict(fit, re.form=~0, type="response", newdata=nd)
	if(length(fit@optinfo$conv$lme4)>0) v0[] <- NA
	v0
}, mc.cores=nc)

ifitm <- do.call("rbind", ifitr.bs)
ifitq <- apply(ifitm, 2, quantile, c(0.025, 0.25, 0.50, 0.75, 0.975), na.rm=TRUE)
ifitq <- cbind(Year=as.numeric(yrs), as.data.frame(t(ifitq)))

# plot
png("figNEAIR.png", 600, 400)
ggplot(ifitq, aes(x=Year)) +
  geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), fill="gray", alpha=0.60) +
  geom_ribbon(aes(ymin = `25%`, ymax = `75%`), fill="gray", alpha=0.95) +
  geom_line(aes(y=`50%`)) + expand_limits(y=0) +
  geom_point(aes(x=Year[1], y=`50%`[1])) +
  geom_point(aes(x=Year[length(Year)], y=`50%`[length(`50%`)]), size=2) +
  ylab("Recruitment (millions)") + 
  xlab("") +
  theme(legend.position = "none") + sc + th
dev.off()

# table
df0 <- t(ifitq)[-1,]
colnames(df0) <- ifitq[,1]
write.csv(df0, file="tabNEAIR.csv")

#--------------------------------------------------------------------
# MED
#--------------------------------------------------------------------
idx <- sam$year!=2015 & !is.na(sam$R)
df0 <- sam[idx,]
df0 <- transform(df0, year=factor(df0$year), R=R/1000)
yrs <- levels(df0$year)
nd <- data.frame(year=factor(yrs))
# model
mfitr <- glmer(R ~ year + (1|stk), data =  df0, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap"))
runDiagsME(mfitr, "stk", df0, "diagMEDIR.pdf", nc, nd) 

# bootstrap
stk <- unique(df0$stk)
mfitr.bs <- split(1:it, 1:it) 
mfitr.bs <- mclapply(mfitr.bs, function(x){
	stk <- sample(stk, replace=TRUE)
	df1 <- df0[0,] 
	for(i in stk) df1 <- rbind(df1, subset(df0, stk==i))
	fit <- glmer(R ~ year + (1|stk), data = df1, family = Gamma("log"),
    control=glmerControl(optimizer="nlminbwrap")) 
	v0 <- predict(fit, re.form=~0, type="response", newdata=nd)
	if(length(fit@optinfo$conv$lme4)>0) v0[] <- NA
	v0
}, mc.cores=nc)
mfitm <- do.call("rbind", mfitr.bs)
mfitq <- apply(mfitm, 2, quantile, c(0.025, 0.25, 0.50, 0.75, 0.975), na.rm=TRUE)
mfitq <- cbind(year=as.numeric(yrs), as.data.frame(t(mfitq)))

# plot
png("figMedIR.png", 600, 400)
ggplot(mfitq, aes(x=year)) + 
	geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), fill="gray", alpha=0.60) + 
	geom_ribbon(aes(ymin = `25%`, ymax = `75%`), fill="gray", alpha=0.95) + 
	geom_line(aes(y=`50%`)) + 
	expand_limits(y=0) + 
	geom_point(aes(x=year[length(year)], y=`50%`[length(`50%`)]), size=2) + 
	ylab("Recruitment (millions)") + 
	xlab("") + 
	theme(legend.position = "none") + 
	scale_x_continuous(breaks=2003:2014) +
	th
dev.off()

# table
df0 <- t(mfitq)[-1,]
colnames(df0) <- mfitq[,1]
write.csv(df0, file="tabMEDIR.csv")

#====================================================================
# SSB/Bpa
#====================================================================
idx <- !is.na(saeu$indBpa)
df0 <- saeu[idx,]
df0$Year <- factor(df0$Year)
yrs <- levels(df0$Year)
nd <- data.frame(Year=factor(yrs))
# fit
ifitbpa <- glmer(indBpa ~ Year + (1|FishStock), data =  df0, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
runDiagsME(ifitbpa, "FishStock", df0, "diagNEAIbpa.pdf", nc, nd) 

# bootstrap
stk <- unique(df0$FishStock)
ifitbpa.bs <- split(1:it, 1:it) 
ifitbpa.bs <- mclapply(ifitbpa.bs, function(x){
	stk <- sample(stk, replace=TRUE)
	df1 <- df0[0,] 
	for(i in stk) df1 <- rbind(df1, subset(df0, FishStock==i))
	fit <- glmer(indBpa ~ Year + (1|FishStock), data = df1, family = Gamma("log"), control=glmerControl(optimizer="nlminbwrap")) 
	v0 <- predict(fit, re.form=~0, type="response", newdata=nd)
	if(length(fit@optinfo$conv$lme4)>0) v0[] <- NA
	v0
}, mc.cores=nc)
ifitm <- do.call("rbind", ifitbpa.bs)
ifitq <- apply(ifitm, 2, quantile, c(0.025, 0.25, 0.50, 0.75, 0.975), na.rm=TRUE)
ifitq <- cbind(Year=as.numeric(yrs), as.data.frame(t(ifitq)))

# plot
png("figNEAIBpa.png", 600, 400)
ggplot(ifitq, aes(x=Year)) +
  geom_ribbon(aes(ymin = `2.5%`, ymax = `97.5%`), fill="gray", alpha=0.60) +
  geom_ribbon(aes(ymin = `25%`, ymax = `75%`), fill="gray", alpha=0.95) +
  geom_line(aes(y=`50%`)) + 
  ylim(c(0,2)) + 
  geom_point(aes(x=Year[1], y=`50%`[1])) +
  geom_point(aes(x=Year[length(Year)], y=`50%`[length(`50%`)]), size=2) +
  geom_hline(yintercept = 1, linetype=2) +
  ylab(expression(B/B[PA])) + xlab("") +
  theme(legend.position = "none") + sc + th
dev.off()

# table
df0 <- t(ifitq)[-1,]
colnames(df0) <- ifitq[,1]
write.csv(df0, file="tabNEAIBpa.csv")


